@@ -283,6 +283,10 @@ main(int argc, char *const *argv)
 
     ngx_os_status(cycle->log);
 
+    if (ngx_crc32_init(cycle->pool) != NGX_OK) {
+        return 1;
+    }
+
     ngx_cycle = cycle;
 
     ccf = (ngx_core_conf_t *) ngx_get_conf(cycle->conf_ctx, ngx_core_module);
@@ -352,7 +356,8 @@ ngx_add_inherited_sockets(ngx_cycle_t *cycle)
                   "using inherited sockets from \"%s\"", inherited);
 
     if (ngx_array_init(&cycle->listening, cycle->pool, 10,
-                                         sizeof(ngx_listening_t)) == NGX_ERROR)
+                       sizeof(ngx_listening_t))
+        == NGX_ERROR)
     {
         return NGX_ERROR;
     }
