@@ -57,6 +57,7 @@ typedef void (*ngx_connection_handler_pt)(ngx_connection_t *c);
 #include <ngx_file.h>
 #include <ngx_files.h>
 #include <ngx_crc.h>
+#include <ngx_crc32.h>
 #if (NGX_PCRE)
 #include <ngx_regex.h>
 #endif
@@ -67,9 +68,6 @@ typedef void (*ngx_connection_handler_pt)(ngx_connection_t *c);
 #include <ngx_event_openssl.h>
 #endif
 #include <ngx_inet.h>
-#if (NGX_HAVE_UNIX_DOMAIN)
-#include <ngx_unix_domain.h>
-#endif
 #include <ngx_cycle.h>
 #include <ngx_process_cycle.h>
 #include <ngx_conf_file.h>
