@@ -20,10 +20,10 @@ typedef struct {
 } ngx_http_addition_ctx_t;
 
 
-static ngx_int_t ngx_http_addition_filter_init(ngx_cycle_t *cycle);
 static void *ngx_http_addition_create_conf(ngx_conf_t *cf);
 static char *ngx_http_addition_merge_conf(ngx_conf_t *cf, void *parent,
     void *child);
+static ngx_int_t ngx_http_addition_filter_init(ngx_conf_t *cf);
 
 
 static ngx_command_t  ngx_http_addition_commands[] = {
@@ -48,7 +48,7 @@ static ngx_command_t  ngx_http_addition_commands[] = {
 
 static ngx_http_module_t  ngx_http_addition_filter_module_ctx = {
     NULL,                                  /* preconfiguration */
-    NULL,                                  /* postconfiguration */
+    ngx_http_addition_filter_init,         /* postconfiguration */
 
     NULL,                                  /* create main configuration */
     NULL,                                  /* init main configuration */
@@ -67,7 +67,7 @@ ngx_module_t  ngx_http_addition_filter_module = {
     ngx_http_addition_commands,            /* module directives */
     NGX_HTTP_MODULE,                       /* module type */
     NULL,                                  /* init master */
-    ngx_http_addition_filter_init,         /* init module */
+    NULL,                                  /* init module */
     NULL,                                  /* init process */
     NULL,                                  /* init thread */
     NULL,                                  /* exit thread */
@@ -124,6 +124,7 @@ ngx_http_addition_body_filter(ngx_http_request_t *r, ngx_chain_t *in)
     ngx_int_t                  rc;
     ngx_uint_t                 last;
     ngx_chain_t               *cl;
+    ngx_http_request_t        *sr;
     ngx_http_addition_ctx_t   *ctx;
     ngx_http_addition_conf_t  *conf;
 
@@ -143,14 +144,19 @@ ngx_http_addition_body_filter(ngx_http_request_t *r, ngx_chain_t *in)
         ctx->before_body_sent = 1;
 
         if (conf->before_body.len) {
-            if (ngx_http_subrequest(r, &conf->before_body, NULL, NULL, 0)
-                == NGX_ERROR)
-            {
-                return NGX_ERROR;
+            rc = ngx_http_subrequest(r, &conf->before_body, NULL, &sr, NULL, 0);
+
+            if (rc == NGX_ERROR || rc == NGX_DONE) {
+                return rc;
             }
         }
     }
 
+    if (conf->after_body.len == 0) {
+        ngx_http_set_ctx(r, NULL, ngx_http_addition_filter_module);
+        return ngx_http_next_body_filter(r, in);
+    }
+
     last = 0;
 
     for (cl = in; cl; cl = cl->next) {
@@ -167,8 +173,10 @@ ngx_http_addition_body_filter(ngx_http_request_t *r, ngx_chain_t *in)
         return rc;
     }
 
-    if (ngx_http_subrequest(r, &conf->after_body, NULL, NULL, 0) == NGX_ERROR) {
-        return NGX_ERROR;
+    rc = ngx_http_subrequest(r, &conf->after_body, NULL, &sr, NULL, 0);
+
+    if (rc == NGX_ERROR || rc == NGX_DONE) {
+        return rc;
     }
 
     ngx_http_set_ctx(r, NULL, ngx_http_addition_filter_module);
@@ -178,7 +186,7 @@ ngx_http_addition_body_filter(ngx_http_request_t *r, ngx_chain_t *in)
 
 
 static ngx_int_t
-ngx_http_addition_filter_init(ngx_cycle_t *cycle)
+ngx_http_addition_filter_init(ngx_conf_t *cf)
 {
     ngx_http_next_header_filter = ngx_http_top_header_filter;
     ngx_http_top_header_filter = ngx_http_addition_header_filter;
