@@ -135,7 +135,7 @@ static ngx_conf_bitmask_t  ngx_http_proxy_next_upstream_masks[] = {
 static ngx_command_t  ngx_http_proxy_commands[] = {
 
     { ngx_string("proxy_pass"),
-      NGX_HTTP_LOC_CONF|NGX_HTTP_LIF_CONF|NGX_CONF_TAKE1,
+      NGX_HTTP_LOC_CONF|NGX_HTTP_LIF_CONF|NGX_HTTP_LMT_CONF|NGX_CONF_TAKE1,
       ngx_http_proxy_pass,
       NGX_HTTP_LOC_CONF_OFFSET,
       0,
@@ -1397,8 +1397,11 @@ ngx_http_proxy_rewrite_redirect_vars(ngx_http_request_t *r, ngx_table_elt_t *h,
     e.ip = pr->replacement.vars.lengths;
     e.request = r;
 
-    for (len = prefix; *(uintptr_t *) e.ip; len += lcode(&e)) {
+    len = prefix + h->value.len - pr->redirect.len;
+
+    while (*(uintptr_t *) e.ip) {
         lcode = *(ngx_http_script_len_code_pt *) e.ip;
+        len += lcode(&e);
     }
 
     data = ngx_palloc(r->pool, len);
@@ -1418,6 +1421,9 @@ ngx_http_proxy_rewrite_redirect_vars(ngx_http_request_t *r, ngx_table_elt_t *h,
         code(&e);
     }
 
+    ngx_memcpy(e.pos, h->value.data + prefix + pr->redirect.len,
+               h->value.len - pr->redirect.len - prefix);
+
     h->value.len = len;
     h->value.data = data;
 
@@ -1817,7 +1823,7 @@ ngx_http_proxy_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
     hash.hash = &conf->upstream.hide_headers_hash;
     hash.key = ngx_hash_key_lc;
     hash.max_size = 512;
-    hash.bucket_size = ngx_cacheline_size;
+    hash.bucket_size = ngx_align(64, ngx_cacheline_size);
     hash.name = "proxy_hide_headers_hash";
     hash.pool = cf->pool;
     hash.temp_pool = NULL;
@@ -2099,32 +2105,6 @@ peers:
         return NGX_CONF_ERROR;
     }
 
-
-#if 0
-    conf->headers_set_hash = ngx_pcalloc(cf->pool, sizeof(ngx_hash0_t));
-    if (conf->headers_set_hash == NULL) {
-        return NGX_CONF_ERROR;
-    }
-
-    conf->headers_set_hash->max_size = 100;
-    conf->headers_set_hash->bucket_limit = 1;
-    conf->headers_set_hash->bucket_size = sizeof(ngx_str_t);
-    conf->headers_set_hash->name = "proxy_headers";
-
-    if (ngx_hash0_init(conf->headers_set_hash, cf->pool,
-                       conf->headers_names->elts, conf->headers_names->nelts)
-        != NGX_OK)
-    {
-        return NGX_CONF_ERROR;
-    }
-
-    ngx_log_debug2(NGX_LOG_DEBUG_HTTP, cf->log, 0,
-                   "proxy_headers hash size: %ui, "
-                   "max buckets per entry: %ui",
-                   conf->headers_set_hash->hash_size,
-                   conf->headers_set_hash->min_buckets);
-#endif
-
     return NGX_CONF_OK;
 }
 
@@ -2227,8 +2207,9 @@ ngx_http_proxy_pass(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
         if (plcf->upstream.uri.len) {
             ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
                                "\"proxy_pass\" may not have URI part in "
-                               "location given by regular expression or "
-                               "inside the \"if\" statement");
+                               "location given by regular expression, "
+                               "or inside the \"if\" statement, "
+                               "or inside the \"limit_except\" block");
             return NGX_CONF_ERROR;
         }
 
