@@ -59,7 +59,7 @@ static ngx_int_t ngx_http_userid_add_variables(ngx_conf_t *cf);
 static ngx_int_t ngx_http_userid_variable(ngx_http_request_t *r,
     ngx_http_variable_value_t *v, uintptr_t data);
 
-static ngx_int_t ngx_http_userid_init(ngx_cycle_t *cycle);
+static ngx_int_t ngx_http_userid_init(ngx_conf_t *cf);
 static void *ngx_http_userid_create_conf(ngx_conf_t *cf);
 static char *ngx_http_userid_merge_conf(ngx_conf_t *cf, void *parent,
     void *child);
@@ -162,7 +162,7 @@ static ngx_command_t  ngx_http_userid_commands[] = {
 
 static ngx_http_module_t  ngx_http_userid_filter_module_ctx = {
     ngx_http_userid_add_variables,         /* preconfiguration */
-    NULL,                                  /* postconfiguration */
+    ngx_http_userid_init,                  /* postconfiguration */
 
     NULL,                                  /* create main configuration */
     NULL,                                  /* init main configuration */
@@ -181,7 +181,7 @@ ngx_module_t  ngx_http_userid_filter_module = {
     ngx_http_userid_commands,              /* module directives */
     NGX_HTTP_MODULE,                       /* module type */
     NULL,                                  /* init master */
-    ngx_http_userid_init,                  /* init module */
+    NULL,                                  /* init module */
     NULL,                                  /* init process */
     NULL,                                  /* init thread */
     NULL,                                  /* exit thread */
@@ -348,10 +348,12 @@ ngx_http_userid_set_uid(ngx_http_request_t *r, ngx_http_userid_ctx_t *ctx,
                 if (r->in_addr == 0) {
                     slen = sizeof(struct sockaddr_in);
                     if (getsockname(r->connection->fd,
-                                    (struct sockaddr *) &sin, &slen) == -1)
+                                    (struct sockaddr *) &sin, &slen)
+                        == -1)
                     {
-                        ngx_log_error(NGX_LOG_CRIT, r->connection->log,
-                                      ngx_socket_errno, "getsockname() failed");
+                        ngx_connection_error(r->connection, ngx_socket_errno,
+                                             "getsockname() failed");
+                        return NGX_ERROR;
                     }
 
                     r->in_addr = sin.sin_addr.s_addr;
@@ -622,16 +624,6 @@ ngx_http_userid_variable(ngx_http_request_t *r, ngx_http_variable_value_t *v,
 }
 
 
-static ngx_int_t
-ngx_http_userid_init(ngx_cycle_t *cycle)
-{
-    ngx_http_next_header_filter = ngx_http_top_header_filter;
-    ngx_http_top_header_filter = ngx_http_userid_filter;
-
-    return NGX_OK;
-}
-
-
 static void *
 ngx_http_userid_create_conf(ngx_conf_t *cf)
 {
@@ -693,6 +685,16 @@ ngx_http_userid_merge_conf(ngx_conf_t *cf, void *parent, void *child)
 }
 
 
+static ngx_int_t
+ngx_http_userid_init(ngx_conf_t *cf)
+{
+    ngx_http_next_header_filter = ngx_http_top_header_filter;
+    ngx_http_top_header_filter = ngx_http_userid_filter;
+
+    return NGX_OK;
+}
+
+
 static char *
 ngx_http_userid_domain(ngx_conf_t *cf, void *post, void *data)
 {
