@@ -11,6 +11,7 @@
 #include <ngx_config.h>
 #include <ngx_core.h>
 #include <ngx_http.h>
+#include <nginx.h>
 
 #include <EXTERN.h>
 #include <perl.h>
@@ -24,6 +25,9 @@ typedef struct {
     ngx_str_t                 filename;
     ngx_str_t                 redirect_uri;
     ngx_str_t                 redirect_args;
+    SV                       *next;
+
+    ngx_uint_t                done;   /* unsigned  done:1; */
 
 #if (NGX_HTTP_SSI)
     ngx_http_ssi_ctx_t       *ssi;
@@ -47,4 +51,7 @@ extern ngx_module_t  ngx_http_perl_module;
 extern void  boot_DynaLoader(pTHX_ CV* cv);
 
 
+void ngx_http_perl_handle_request(ngx_http_request_t *r);
+
+
 #endif /* _NGX_HTTP_PERL_MODULE_H_INCLUDED_ */
