@@ -23,7 +23,6 @@ typedef struct {
 #define NGX_HTTP_LOCATION_REGEX           4
 
 
-static void ngx_http_core_run_phases(ngx_http_request_t *r);
 static ngx_int_t ngx_http_core_find_location(ngx_http_request_t *r,
     ngx_array_t *locations, size_t len);
 
@@ -474,6 +473,8 @@ ngx_module_t  ngx_http_core_module = {
 void
 ngx_http_handler(ngx_http_request_t *r)
 {
+    ngx_http_core_main_conf_t  *cmcf;
+
     r->connection->log->action = NULL;
 
     r->connection->unexpected_eof = 0;
@@ -500,8 +501,8 @@ ngx_http_handler(ngx_http_request_t *r)
         if (r->keepalive && r->headers_in.msie && r->method == NGX_HTTP_POST) {
 
             /*
-             * MSIE may wait for some time if the response for
-             * the POST request is sent over the keepalive connection
+             * MSIE may wait for some time if an response for
+             * a POST request was sent over a keepalive connection
              */
 
             r->keepalive = 0;
@@ -513,174 +514,87 @@ ngx_http_handler(ngx_http_request_t *r)
         } else {
             r->lingering_close = 0;
         }
+
+        r->phase_handler = 0;
+
+    } else {
+        cmcf = ngx_http_get_module_main_conf(r, ngx_http_core_module);
+        r->phase_handler = cmcf->phase_engine.server_rewrite_index;
     }
 
-    r->write_event_handler = ngx_http_core_run_phases;
+    if (r->unparsed_uri.len) {
+        r->valid_unparsed_uri = 1;
+    }
 
-    r->valid_unparsed_uri = 1;
     r->valid_location = 1;
-    r->uri_changed = 1;
-
-    r->phase = (!r->internal) ? NGX_HTTP_POST_READ_PHASE:
-                                NGX_HTTP_SERVER_REWRITE_PHASE;
-    r->phase_handler = 0;
 
+    r->write_event_handler = ngx_http_core_run_phases;
     ngx_http_core_run_phases(r);
 }
 
 
-static void
+void
 ngx_http_core_run_phases(ngx_http_request_t *r)
 {
     ngx_int_t                   rc;
-    ngx_str_t                   path;
-    ngx_http_handler_pt        *h;
-    ngx_http_core_loc_conf_t   *clcf;
+    ngx_http_phase_handler_t   *ph;
     ngx_http_core_main_conf_t  *cmcf;
 
-    ngx_log_debug0(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
-                   "http phase handler");
-
     cmcf = ngx_http_get_module_main_conf(r, ngx_http_core_module);
 
-    for (/* void */; r->phase < NGX_HTTP_LOG_PHASE; r->phase++) {
+    ph = cmcf->phase_engine.handlers;
 
-        if (r->phase == NGX_HTTP_REWRITE_PHASE + 1 && r->uri_changed) {
+    while (ph[r->phase_handler].checker) {
 
-            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
-                           "uri changes: %d", r->uri_changes);
+        rc = ph[r->phase_handler].checker(r, &ph[r->phase_handler]);
 
-            /*
-             * gcc before 3.3 compiles the broken code for
-             *     if (r->uri_changes-- == 0)
-             * if the r->uri_changes is defined as
-             *     unsigned  uri_changes:4
-             */
-
-            r->uri_changes--;
-
-            if (r->uri_changes == 0) {
-                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
-                              "rewrite or internal redirection cycle "
-                              "while processing \"%V\"", &r->uri);
-                ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
-                return;
-            }
-
-            r->phase = NGX_HTTP_FIND_CONFIG_PHASE;
-        }
-
-        if (r->phase == NGX_HTTP_ACCESS_PHASE && r != r->main) {
-            continue;
-        }
-
-        if (r->phase == NGX_HTTP_CONTENT_PHASE && r->content_handler) {
-            r->write_event_handler = ngx_http_request_empty_handler;
-            ngx_http_finalize_request(r, r->content_handler(r));
-            return;
-        }
-
-        h = cmcf->phases[r->phase].handlers.elts;
-        for (r->phase_handler = cmcf->phases[r->phase].handlers.nelts - 1;
-             r->phase_handler >= 0;
-             r->phase_handler--)
-        {
-            rc = h[r->phase_handler](r);
-
-            if (rc == NGX_DONE) {
-
-                /*
-                 * we should never use r here because
-                 * it may point to already freed data
-                 */
-
-                return;
-            }
-
-            if (rc == NGX_DECLINED) {
-                continue;
-            }
-
-            if (r->phase == NGX_HTTP_ACCESS_PHASE) {
-                clcf = ngx_http_get_module_loc_conf(r, ngx_http_core_module);
-
-                if (clcf->satisfy_any) {
-
-                    if (rc == NGX_OK) {
-                        r->access_code = 0;
-
-                        if (r->headers_out.www_authenticate) {
-                            r->headers_out.www_authenticate->hash = 0;
-                        }
-
-                        break;
-                    }
-
-                    if (rc == NGX_HTTP_FORBIDDEN || rc == NGX_HTTP_UNAUTHORIZED)
-                    {
-                        r->access_code = rc;
-
-                        continue;
-                    }
-                }
-            }
-
-            if (rc >= NGX_HTTP_SPECIAL_RESPONSE
-                || rc == NGX_HTTP_NO_CONTENT
-                || rc == NGX_ERROR)
-            {
-                ngx_http_finalize_request(r, rc);
-                return;
-            }
-
-            if (r->phase == NGX_HTTP_CONTENT_PHASE) {
-                ngx_http_finalize_request(r, rc);
-                return;
-            }
-
-            if (rc == NGX_AGAIN) {
-                return;
-            }
-
-            if (rc == NGX_OK && cmcf->phases[r->phase].type == NGX_OK) {
-                break;
-            }
-
-        }
-
-        if (r->phase == NGX_HTTP_ACCESS_PHASE && r->access_code) {
-
-            if (r->access_code == NGX_HTTP_FORBIDDEN) {
-                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
-                              "access forbidden by rule");
-            }
-
-            ngx_http_finalize_request(r, r->access_code);
+        if (rc == NGX_OK) {
             return;
         }
     }
-
-    /* no content handler was found */
-
-    if (r->uri.data[r->uri.len - 1] == '/' && !r->zero_in_uri) {
-
-        if (ngx_http_map_uri_to_path(r, &path, 0) != NULL) {
-            ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
-                          "directory index of \"%V\" is forbidden", &path);
-        }
-
-        ngx_http_finalize_request(r, NGX_HTTP_FORBIDDEN);
-        return;
-    }
-
-    ngx_log_error(NGX_LOG_ERR, r->connection->log, 0, "no handler found");
-
-    ngx_http_finalize_request(r, NGX_HTTP_NOT_FOUND);
 }
 
 
 ngx_int_t
-ngx_http_find_location_config(ngx_http_request_t *r)
+ngx_http_core_generic_phase(ngx_http_request_t *r, ngx_http_phase_handler_t *ph)
+{
+    ngx_int_t  rc;
+
+    /*
+     * generic phase checker,
+     * used by the post read, server rewrite, rewrite, and pre-access phases
+     */
+
+    ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                   "generic phase: %ui", r->phase_handler);
+
+    rc = ph->handler(r);
+
+    if (rc == NGX_OK) {
+        r->phase_handler = ph->next;
+        return NGX_AGAIN;
+    }
+
+    if (rc == NGX_DECLINED) {
+        r->phase_handler++;
+        return NGX_AGAIN;
+    }
+
+    if (rc == NGX_AGAIN || rc == NGX_DONE) {
+        return NGX_OK;
+    }
+
+    /* rc == NGX_ERROR || rc == NGX_HTTP_...  */
+
+    ngx_http_finalize_request(r, rc);
+
+    return NGX_OK;
+}
+
+
+ngx_int_t
+ngx_http_core_find_config_phase(ngx_http_request_t *r,
+    ngx_http_phase_handler_t *ph)
 {
     ngx_int_t                  rc;
     ngx_http_core_loc_conf_t  *clcf;
@@ -694,13 +608,15 @@ ngx_http_find_location_config(ngx_http_request_t *r)
     rc = ngx_http_core_find_location(r, &cscf->locations, 0);
 
     if (rc == NGX_HTTP_INTERNAL_SERVER_ERROR) {
-        return rc;
+        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
+        return NGX_OK;
     }
 
     clcf = ngx_http_get_module_loc_conf(r, ngx_http_core_module);
 
     if (!r->internal && clcf->internal) {
-        return NGX_HTTP_NOT_FOUND;
+        ngx_http_finalize_request(r, NGX_HTTP_NOT_FOUND);
+        return NGX_OK;
     }
 
     ngx_log_debug2(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
@@ -723,14 +639,16 @@ ngx_http_find_location_config(ngx_http_request_t *r)
                       "client intented to send too large body: %O bytes",
                       r->headers_in.content_length_n);
 
-        return NGX_HTTP_REQUEST_ENTITY_TOO_LARGE;
+        ngx_http_finalize_request(r, NGX_HTTP_REQUEST_ENTITY_TOO_LARGE);
+        return NGX_OK;
     }
 
 
     if (rc == NGX_HTTP_LOCATION_AUTO_REDIRECT) {
         r->headers_out.location = ngx_list_push(&r->headers_out.headers);
         if (r->headers_out.location == NULL) {
-            return NGX_HTTP_INTERNAL_SERVER_ERROR;
+            ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
+            return NGX_OK;
         }
 
         /*
@@ -740,9 +658,191 @@ ngx_http_find_location_config(ngx_http_request_t *r)
 
         r->headers_out.location->value = clcf->name;
 
-        return NGX_HTTP_MOVED_PERMANENTLY;
+        ngx_http_finalize_request(r, NGX_HTTP_MOVED_PERMANENTLY);
+        return NGX_OK;
     }
 
+    r->phase_handler++;
+    return NGX_AGAIN;
+}
+
+
+ngx_int_t
+ngx_http_core_post_rewrite_phase(ngx_http_request_t *r,
+    ngx_http_phase_handler_t *ph)
+{
+    ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                   "post rewrite phase: %ui", r->phase_handler);
+
+    if (!r->uri_changed) {
+        r->phase_handler++;
+        return NGX_AGAIN;
+    }
+
+    ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                   "uri changes: %d", r->uri_changes);
+
+    /*
+     * gcc before 3.3 compiles the broken code for
+     *     if (r->uri_changes-- == 0)
+     * if the r->uri_changes is defined as
+     *     unsigned  uri_changes:4
+     */
+
+    r->uri_changes--;
+
+    if (r->uri_changes == 0) {
+        ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                      "rewrite or internal redirection cycle "
+                      "while processing \"%V\"", &r->uri);
+
+        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
+        return NGX_OK;
+    }
+
+    r->phase_handler = ph->next;
+
+    return NGX_AGAIN;
+}
+
+
+ngx_int_t
+ngx_http_core_access_phase(ngx_http_request_t *r, ngx_http_phase_handler_t *ph)
+{
+    ngx_int_t                  rc;
+    ngx_http_core_loc_conf_t  *clcf;
+
+    if (r != r->main) {
+        r->phase_handler = ph->next;
+        return NGX_AGAIN;
+    }
+
+    ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                   "access phase: %ui", r->phase_handler);
+
+    rc = ph->handler(r);
+
+    if (rc == NGX_DECLINED) {
+        r->phase_handler++;
+        return NGX_AGAIN;
+    }
+
+    if (rc == NGX_AGAIN || rc == NGX_DONE) {
+        return NGX_OK;
+    }
+
+    clcf = ngx_http_get_module_loc_conf(r, ngx_http_core_module);
+
+    if (clcf->satisfy_any == 0) {
+
+        if (rc == NGX_OK) {
+            r->phase_handler++;
+            return NGX_AGAIN;
+        }
+
+    } else {
+        if (rc == NGX_OK) {
+            r->access_code = 0;
+
+            if (r->headers_out.www_authenticate) {
+                r->headers_out.www_authenticate->hash = 0;
+            }
+
+            r->phase_handler = ph->next;
+            return NGX_AGAIN;
+        }
+
+        if (rc == NGX_HTTP_FORBIDDEN || rc == NGX_HTTP_UNAUTHORIZED) {
+            r->access_code = rc;
+
+            r->phase_handler++;
+            return NGX_AGAIN;
+        }
+    }
+
+    /* rc == NGX_ERROR || rc == NGX_HTTP_...  */
+
+    ngx_http_finalize_request(r, rc);
+    return NGX_OK;
+}
+
+
+ngx_int_t
+ngx_http_core_post_access_phase(ngx_http_request_t *r,
+    ngx_http_phase_handler_t *ph)
+{
+    ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                   "post access phase: %ui", r->phase_handler);
+
+    if (r->access_code) {
+
+        if (r->access_code == NGX_HTTP_FORBIDDEN) {
+            ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                          "access forbidden by rule");
+        }
+
+        ngx_http_finalize_request(r, r->access_code);
+        return NGX_OK;
+    }
+
+    r->phase_handler++;
+    return NGX_AGAIN;
+}
+
+
+ngx_int_t
+ngx_http_core_content_phase(ngx_http_request_t *r,
+    ngx_http_phase_handler_t *ph)
+{
+    size_t     root;
+    ngx_int_t  rc;
+    ngx_str_t  path;
+
+    if (r->content_handler) {
+        r->write_event_handler = ngx_http_request_empty_handler;
+        ngx_http_finalize_request(r, r->content_handler(r));
+        return NGX_OK;
+    }
+
+    ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                   "content phase: %ui", r->phase_handler);
+
+    rc = ph->handler(r);
+
+    if (rc == NGX_DONE) {
+        return NGX_OK;
+    }
+
+    if (rc != NGX_DECLINED) {
+        ngx_http_finalize_request(r, rc);
+        return NGX_OK;
+    }
+
+    /* rc == NGX_DECLINED */
+
+    ph++;
+
+    if (ph->checker) {
+        r->phase_handler++;
+        return NGX_AGAIN;
+    }
+
+    /* no content handler was found */
+
+    if (r->uri.data[r->uri.len - 1] == '/' && !r->zero_in_uri) {
+
+        if (ngx_http_map_uri_to_path(r, &path, &root, 0) != NULL) {
+            ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                          "directory index of \"%V\" is forbidden", &path);
+        }
+
+        ngx_http_finalize_request(r, NGX_HTTP_FORBIDDEN);
+        return NGX_OK;
+    }
+
+    ngx_log_error(NGX_LOG_ERR, r->connection->log, 0, "no handler found");
+
+    ngx_http_finalize_request(r, NGX_HTTP_NOT_FOUND);
     return NGX_OK;
 }
 
@@ -1058,7 +1158,7 @@ ngx_http_output_filter(ngx_http_request_t *r, ngx_chain_t *in)
 
 u_char *
 ngx_http_map_uri_to_path(ngx_http_request_t *r, ngx_str_t *path,
-    size_t reserved)
+    size_t *root_length, size_t reserved)
 {
     u_char                    *last;
     size_t                     alias;
@@ -1079,7 +1179,7 @@ ngx_http_map_uri_to_path(ngx_http_request_t *r, ngx_str_t *path,
 
     if (clcf->root_lengths == NULL) {
 
-        r->root_length = clcf->root.len;
+        *root_length = clcf->root.len;
 
         path->len = clcf->root.len + reserved;
 
@@ -1102,8 +1202,8 @@ ngx_http_map_uri_to_path(ngx_http_request_t *r, ngx_str_t *path,
             return NULL;
         }
 
-        r->root_length = path->len - reserved;
-        last = path->data + r->root_length;
+        *root_length = path->len - reserved;
+        last = path->data + *root_length;
     }
 
     last = ngx_cpystrn(last, r->uri.data + alias, r->uri.len - alias + 1);
@@ -1184,7 +1284,8 @@ ngx_http_auth_basic_user(ngx_http_request_t *r)
 
 ngx_int_t
 ngx_http_subrequest(ngx_http_request_t *r,
-    ngx_str_t *uri, ngx_str_t *args, ngx_chain_t *out, ngx_uint_t flags)
+    ngx_str_t *uri, ngx_str_t *args, ngx_http_request_t **psr,
+    ngx_http_post_subrequest_t *ps, ngx_uint_t flags)
 {
     ngx_connection_t              *c;
     ngx_http_request_t            *sr;
@@ -1197,6 +1298,7 @@ ngx_http_subrequest(ngx_http_request_t *r,
     if (r->main->subrequests == 0) {
         ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
                       "subrequests cycle while processing \"%V\"", uri);
+        r->main->subrequests = 1;
         return NGX_ERROR;
     }
 
@@ -1241,7 +1343,6 @@ ngx_http_subrequest(ngx_http_request_t *r,
 
     sr->method = NGX_HTTP_GET;
     sr->http_version = r->http_version;
-    sr->http_major = r->http_minor;
 
     sr->request_line = r->request_line;
     sr->uri = *uri;
@@ -1253,9 +1354,8 @@ ngx_http_subrequest(ngx_http_request_t *r,
     ngx_log_debug2(NGX_LOG_DEBUG_HTTP, c->log, 0,
                    "http subrequest \"%V?%V\"", uri, &sr->args);
 
-    if (flags & NGX_HTTP_ZERO_IN_URI) {
-        sr->zero_in_uri = 1;
-    }
+    sr->zero_in_uri = (flags & NGX_HTTP_ZERO_IN_URI) != 0;
+    sr->subrequest_in_memory = (flags & NGX_HTTP_SUBREQUEST_IN_MEMORY) != 0;
 
     sr->unparsed_uri = r->unparsed_uri;
     sr->method_name = r->method_name;
@@ -1265,9 +1365,9 @@ ngx_http_subrequest(ngx_http_request_t *r,
         return NGX_ERROR;
     }
 
-    sr->out = out;
     sr->main = r->main;
     sr->parent = r;
+    sr->post_subrequest = ps;
     sr->read_event_handler = ngx_http_request_empty_handler;
     sr->write_event_handler = ngx_http_request_empty_handler;
 
@@ -1318,6 +1418,10 @@ ngx_http_subrequest(ngx_http_request_t *r,
         ngx_log_debug2(NGX_LOG_DEBUG_HTTP, c->log, 0,
                        "http subrequest done \"%V?%V\"", uri, &sr->args);
 
+        r->main->subrequests++;
+
+        *psr = sr;
+
         if (sr->fast_subrequest) {
             sr->fast_subrequest = 0;
 
@@ -1329,7 +1433,7 @@ ngx_http_subrequest(ngx_http_request_t *r,
         return NGX_AGAIN;
     }
 
-    return NGX_OK;
+    return NGX_DONE;
 }
 
 
@@ -1916,7 +2020,7 @@ ngx_http_core_create_srv_conf(ngx_conf_t *cf)
         return NGX_CONF_ERROR;
     }
 
-    if (ngx_array_init(&cscf->server_names, cf->pool, 4,
+    if (ngx_array_init(&cscf->server_names, cf->temp_pool, 4,
                        sizeof(ngx_http_server_name_t))
         == NGX_ERROR)
     {
@@ -2270,12 +2374,10 @@ ngx_http_core_listen(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 {
     ngx_http_core_srv_conf_t *scf = conf;
 
-    char                 *err;
-    ngx_str_t            *value, size;
-    ngx_uint_t            n;
-    struct hostent       *h;
-    ngx_http_listen_t    *ls;
-    ngx_inet_upstream_t   inet_upstream;
+    ngx_str_t          *value, size;
+    ngx_url_t           u;
+    ngx_uint_t          n;
+    ngx_http_listen_t  *ls;
 
     /*
      * TODO: check duplicate 'listen' directives,
@@ -2284,17 +2386,19 @@ ngx_http_core_listen(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 
     value = cf->args->elts;
 
-    ngx_memzero(&inet_upstream, sizeof(ngx_inet_upstream_t));
+    ngx_memzero(&u, sizeof(ngx_url_t));
 
-    inet_upstream.url = value[1];
-    inet_upstream.port_only = 1;
+    u.url = value[1];
+    u.listen = 1;
+    u.default_portn = 80;
 
-    err = ngx_inet_parse_host_port(&inet_upstream);
+    if (ngx_parse_url(cf, &u) != NGX_OK) {
+        if (u.err) {
+            ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
+                               "%s in \"%V\" of the \"listen\" directive",
+                               u.err, &u.url);
+        }
 
-    if (err) {
-        ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
-                           "%s in \"%V\" of the \"listen\" directive",
-                           err, &inet_upstream.url);
         return NGX_CONF_ERROR;
     }
 
@@ -2306,41 +2410,14 @@ ngx_http_core_listen(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
     ngx_memzero(ls, sizeof(ngx_http_listen_t));
 
     ls->family = AF_INET;
-    ls->port = (in_port_t) (inet_upstream.default_port ?
-                            80 : inet_upstream.port);
+    ls->addr = u.addr.in_addr;
+    ls->port = u.portn;
     ls->file_name = cf->conf_file->file.name;
     ls->line = cf->conf_file->line;
     ls->conf.backlog = -1;
     ls->conf.rcvbuf = -1;
     ls->conf.sndbuf = -1;
 
-    if (inet_upstream.host.len == 1 && inet_upstream.host.data[0] == '*') {
-        inet_upstream.host.len = 0;
-    }
-
-    if (inet_upstream.host.len) {
-        inet_upstream.host.data[inet_upstream.host.len] = '\0';
-
-        ls->addr = inet_addr((const char *) inet_upstream.host.data);
-
-        if (ls->addr == INADDR_NONE) {
-            h = gethostbyname((const char *) inet_upstream.host.data);
-
-            if (h == NULL || h->h_addr_list[0] == NULL) {
-                ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
-                                   "can not resolve host \"%s\" "
-                                   "in the \"listen\" directive",
-                                   inet_upstream.host.data);
-                return NGX_CONF_ERROR;
-            }
-
-            ls->addr = *(in_addr_t *)(h->h_addr_list[0]);
-        }
-
-    } else {
-        ls->addr = INADDR_ANY;
-    }
-
     n = ngx_inet_ntop(AF_INET, &ls->addr, ls->conf.addr, INET_ADDRSTRLEN + 6);
     ngx_sprintf(&ls->conf.addr[n], ":%ui", ls->port);
 
@@ -2587,9 +2664,19 @@ ngx_http_core_root(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 
 
 static ngx_http_method_name_t  ngx_methods_names[] = {
-   { "GET",  (uint32_t) ~NGX_HTTP_GET },
-   { "HEAD", (uint32_t) ~NGX_HTTP_HEAD },
-   { "POST", (uint32_t) ~NGX_HTTP_POST },
+   { "GET",       (uint32_t) ~NGX_HTTP_GET },
+   { "HEAD",      (uint32_t) ~NGX_HTTP_HEAD },
+   { "POST",      (uint32_t) ~NGX_HTTP_POST },
+   { "PUT",       (uint32_t) ~NGX_HTTP_PUT },
+   { "DELETE",    (uint32_t) ~NGX_HTTP_DELETE },
+   { "MKCOL",     (uint32_t) ~NGX_HTTP_MKCOL },
+   { "COPY",      (uint32_t) ~NGX_HTTP_COPY },
+   { "MOVE",      (uint32_t) ~NGX_HTTP_MOVE },
+   { "OPTIONS",   (uint32_t) ~NGX_HTTP_OPTIONS },
+   { "PROPFIND" , (uint32_t) ~NGX_HTTP_PROPFIND },
+   { "PROPPATCH", (uint32_t) ~NGX_HTTP_PROPPATCH },
+   { "LOCK",      (uint32_t) ~NGX_HTTP_LOCK },
+   { "UNLOCK",    (uint32_t) ~NGX_HTTP_UNLOCK },
    { NULL, 0 }
 };
 
@@ -2786,7 +2873,7 @@ ngx_http_core_error_page(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 
         err->status = ngx_atoi(value[i].data, value[i].len);
 
-        if (err->status == NGX_ERROR) {
+        if (err->status == NGX_ERROR || err->status == 499) {
             ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
                                "invalid value \"%V\"", &value[i]);
             return NGX_CONF_ERROR;
