@@ -30,6 +30,13 @@
 #define NGX_HTTP_PUT                       0x0010
 #define NGX_HTTP_DELETE                    0x0020
 #define NGX_HTTP_MKCOL                     0x0040
+#define NGX_HTTP_COPY                      0x0080
+#define NGX_HTTP_MOVE                      0x0100
+#define NGX_HTTP_OPTIONS                   0x0200
+#define NGX_HTTP_PROPFIND                  0x0400
+#define NGX_HTTP_PROPPATCH                 0x0800
+#define NGX_HTTP_LOCK                      0x1000
+#define NGX_HTTP_UNLOCK                    0x2000
 
 #define NGX_HTTP_CONNECTION_CLOSE          1
 #define NGX_HTTP_CONNECTION_KEEP_ALIVE     2
@@ -47,7 +54,9 @@
 
 #define NGX_HTTP_PARSE_INVALID_HEADER      13
 
+
 #define NGX_HTTP_ZERO_IN_URI               1
+#define NGX_HTTP_SUBREQUEST_IN_MEMORY      2
 
 
 #define NGX_HTTP_OK                        200
@@ -109,11 +118,11 @@
 #define NGX_HTTP_INSUFFICIENT_STORAGE      507
 
 
-#define NGX_HTTP_LOWLEVEL_BUFFERED         0x000000f0
-#define NGX_HTTP_WRITE_BUFFERED            0x00000010
-#define NGX_HTTP_GZIP_BUFFERED             0x00000020
-#define NGX_HTTP_SSI_BUFFERED              0x00000100
-#define NGX_HTTP_COPY_BUFFERED             0x00000200
+#define NGX_HTTP_LOWLEVEL_BUFFERED         0xf0
+#define NGX_HTTP_WRITE_BUFFERED            0x10
+#define NGX_HTTP_GZIP_BUFFERED             0x20
+#define NGX_HTTP_SSI_BUFFERED              0x01
+#define NGX_HTTP_COPY_BUFFERED             0x02
 
 
 typedef enum {
@@ -184,6 +193,7 @@ typedef struct {
 #if (NGX_HTTP_DAV)
     ngx_table_elt_t                  *depth;
     ngx_table_elt_t                  *destination;
+    ngx_table_elt_t                  *date;
 #endif
 
     ngx_str_t                         user;
@@ -286,6 +296,15 @@ struct ngx_http_cleanup_s {
 };
 
 
+typedef ngx_int_t (*ngx_http_post_subrequest_pt)(ngx_http_request_t *r,
+    void *data, ngx_int_t rc);
+
+typedef struct {
+    ngx_http_post_subrequest_pt       handler;
+    void                             *data;
+} ngx_http_post_subrequest_t;
+
+
 typedef struct ngx_http_postponed_request_s  ngx_http_postponed_request_t;
 
 struct ngx_http_postponed_request_s {
@@ -329,8 +348,6 @@ struct ngx_http_request_s {
 
     ngx_uint_t                        method;
     ngx_uint_t                        http_version;
-    ngx_uint_t                        http_major;
-    ngx_uint_t                        http_minor;
 
     ngx_str_t                         request_line;
     ngx_str_t                         uri;
@@ -345,6 +362,7 @@ struct ngx_http_request_s {
     ngx_http_request_t               *main;
     ngx_http_request_t               *parent;
     ngx_http_postponed_request_t     *postponed;
+    ngx_http_post_subrequest_t       *post_subrequest;
 
     uint32_t                          in_addr;
     ngx_uint_t                        port;
@@ -352,15 +370,12 @@ struct ngx_http_request_s {
     ngx_str_t                         server_name;
     ngx_http_virtual_names_t         *virtual_names;
 
-    ngx_uint_t                        phase;
     ngx_int_t                         phase_handler;
     ngx_http_handler_pt               content_handler;
     ngx_uint_t                        access_code;
 
     ngx_http_variable_value_t        *variables;
 
-    size_t                            root_length;
-
     size_t                            limit_rate;
 
     /* used to learn the Apache compatible response length without a header */
@@ -405,6 +420,7 @@ struct ngx_http_request_s {
     unsigned                          request_body_file_log_level:3;
 
     unsigned                          fast_subrequest:1;
+    unsigned                          subrequest_in_memory:1;
 
     unsigned                          header_timeout_set:1;
 
@@ -430,6 +446,7 @@ struct ngx_http_request_s {
     unsigned                          plain_http:1;
     unsigned                          chunked:1;
     unsigned                          header_only:1;
+    unsigned                          zero_body:1;
     unsigned                          keepalive:1;
     unsigned                          lingering_close:1;
     unsigned                          discard_body:1;
@@ -442,6 +459,8 @@ struct ngx_http_request_s {
     unsigned                          done:1;
     unsigned                          utf8:1;
 
+    unsigned                          buffered:4;
+
     unsigned                          main_filter_need_in_memory:1;
     unsigned                          filter_need_in_memory:1;
     unsigned                          filter_need_temporary:1;
@@ -455,6 +474,7 @@ struct ngx_http_request_s {
     unsigned                          subrequests:8;
 
     /* used to parse HTTP headers */
+
     ngx_uint_t                        state;
     u_char                           *uri_start;
     u_char                           *uri_end;
@@ -474,6 +494,9 @@ struct ngx_http_request_s {
     u_char                           *header_start;
     u_char                           *header_end;
 
+    unsigned                          http_minor:16;
+    unsigned                          http_major:16;
+
     ngx_uint_t                        header_hash;
     ngx_uint_t                        lowcase_index;
     u_char                            lowcase_header[NGX_HTTP_LC_HEADER_LEN];
